==> xdvik-22.74.1.tar.gz: xdvik-22.74.1/texk/xdvik/util.h <==
/*
 * Copyright (c) 2002 xdvik development team
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * PAUL VOJTA OR ANY OTHER AUTHOR OF THIS SOFTWARE BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef UTIL_H_
#define UTIL_H_

#include <stdio.h>
#include "xdvi-config.h"
#include "kpathsea/c-stat.h"
#include "kpathsea/hash.h"

#if HAVE_POLL
# include <poll.h>
# define XIO_IN POLLIN
# define XIO_OUT POLLOUT
#else
# if HAVE_SYS_SELECT_H
#  include <sys/select.h>
# else
#  if HAVE_SELECT_H
#   include <select.h>
#  endif
# endif
# define XIO_IN 1
# define XIO_OUT 2
#endif

/* double linked list */
struct dl_list {
    void *item;
    struct dl_list *prev;
    struct dl_list *next;
};

struct dl_list *dl_list_insert(struct dl_list *list, void *val);
struct dl_list *dl_list_truncate(struct dl_list *list);
void dl_list_remove(struct dl_list **list, void *elem, Boolean (*compare_func)());


#if PS
extern int xdvi_temp_fd(char **tempfilename);
#endif
struct xchild; /* forward declaration */
extern void handle_child_exit(int status, struct xchild *this);
extern char *read_child_error(int fd);
extern Boolean fork_process(_Xconst char *file, char *const argv[]);
extern void print_child_error(void);
extern void prep_fd(int fd, wide_bool noblock);

struct bitmap; /* forward declaration */
extern void alloc_bitmap(struct bitmap *);

extern char *my_realpath(const char *path, char *real);
#ifdef HAVE_REALPATH
#include <limits.h>
# define REALPATH realpath
#else
# define REALPATH my_realpath
#endif

char *expand_homedir(const char *path);
void set_dvi_name(const char *new_filename);

#ifndef KPATHSEA
extern void *xmalloc(unsigned);
extern void *xrealloc(void *, unsigned);
extern char *xstrdup(_Xconst char *);
extern char *xmemdump(_Xconst char *, size_t);
extern void xputenv(_Xconst char *, _Xconst char *);
#endif
extern char *xstrcat(char *str1, _Xconst char *str2);
extern int memicmp(const char *, const char *, size_t);
extern int xpipe(int *);
extern unsigned long num(FILE *, int);
extern long snum(FILE *, int);

extern void xdvi_exit(int);
extern void do_abort(void);

/* various levels of warning/error messages */
extern void xdvi_info(_Xconst char *fmt, ...);
extern void xdvi_warning(_Xconst char *fmt, ...);
extern void xdvi_error(_Xconst char *fmt, ...);
extern void xdvi_fatal(_Xconst char *fmt, ...);
extern void xdvi_abort(_Xconst char *fmt, ...);

extern int pointerlocate(int *, int *);
extern unsigned long parse_debugging_string(const char *arg);
extern unsigned long parse_debugging_option(const char *ptr);

extern int get_avg_font_width(XFontStruct *font);
extern char **split_line(const char *line, char sep, size_t begin, size_t end, size_t *ret_items);
extern char *find_file(const char *filename, struct stat *statbuf);

/*
  hashtable wrapper functions, mostly used by dvi-draw.c to
  map filenames to integers. This uses the hashtable implementation
  from kpathsea, which is reasonably fast.
*/

/*
  We use this dummy wrapper stuct, which we cast to void *, to get integer
  values into/from the hashtable (natively, kpahtsea only supports string
  values).
*/
struct str_int_hash_item {
    int value;
};

extern Boolean find_str_int_hash(hash_table_type *hashtable, const char *key, size_t *val);
void put_str_int_hash(hash_table_type *hashtable, const char *key, size_t val);

/* Various error reporting macros.
   The reasons why these are macros are:
   - possibility to use __FILE__, __LINE__
   - (more importantly:) gcc can't do type checking on generic vararg
     macros, but does so for the printf() functions.
*/

/*
 *	Print an informative message to stdout,
 *	unless the resource `hush_stdout' is set.
 */
#define XDVI_INFO(X) do {						\
	fprintf(stdout, "%s: Info: ", prog == NULL ? "Xdvi" : prog);	\
	fprintf X;							\
	fprintf(stdout, "\n");						\
	fflush(stdout); /* to make sure it doesn't get intermingled with stderr */ \
} while(0)


/*
 *	Print a warning message to stderr.
 *	This can't be shut off by `hush_stdout'.
 */
#define XDVI_WARNING(X) do {						\
	fprintf(stderr, "%s: Warning: ", prog == NULL ? "Xdvi" : prog);	\
	fprintf X;							\
	fprintf(stderr, "\n");						\
} while(0)

/*
 *	Print an error message to stderr.
 *	This can't be shut off by `hush_stdout'.
 */
#define XDVI_ERROR(X) do {						\
	fprintf(stderr, "%s: Error: ", prog == NULL ? "Xdvi" : prog);	\
	fprintf X;							\
	fprintf(stderr, "\n");						\
} while(0)

/*
 *	Print an error message and quit. Use this only with extreme care,
 *	if recovery is really impossible!
 */
#define XDVI_FATAL(X) do {							\
	fprintf(stderr, "%s: Fatal error: ", prog == NULL ? "Xdvi" : prog);	\
	fprintf X;								\
	fprintf(stderr, "\n%s: Exiting.\n", prog == NULL ? "Xdvi" : prog);	\
	xdvi_exit(EXIT_FAILURE);						\
} while(0) 

/*
 *	Print an error message and abort. Use this instead of assertions
 *	if you want to give a more informative message.
 */
#define XDVI_ABORT(X) do {					\
	fprintf(stderr, "%s: %s:%d: Shouldn't happen: ",	\
			prog == NULL ? "Xdvi" : prog,		\
			__FILE__, __LINE__);			\
	fprintf X;						\
	fprintf(stderr, "\n");					\
	do_abort();						\
} while(0)

#endif /* UTIL_H_ */

==> xdvik-22.74.3.tar.gz: xdvik-22.74.3/texk/xdvik/util.h <==
/*
 * Copyright (c) 2002 xdvik development team
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * PAUL VOJTA OR ANY OTHER AUTHOR OF THIS SOFTWARE BE LIABLE FOR ANY CLAIM,
 * DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */

#ifndef UTIL_H_
#define UTIL_H_

#include <stdio.h>
#include "xdvi-config.h"
#include "kpathsea/c-stat.h"
#include "kpathsea/hash.h"
#include "kpathsea/tex-file.h"

#if HAVE_POLL
# include <poll.h>
# define XIO_IN POLLIN
# define XIO_OUT POLLOUT
#else
# if HAVE_SYS_SELECT_H
#  include <sys/select.h>
# else
#  if HAVE_SELECT_H
#   include <select.h>
#  endif
# endif
# define XIO_IN 1
# define XIO_OUT 2
#endif

/* double linked list */
struct dl_list {
    void *item;
    struct dl_list *prev;
    struct dl_list *next;
};

struct dl_list *dl_list_insert(struct dl_list *list, void *val);
struct dl_list *dl_list_truncate(struct dl_list *list);
void dl_list_remove(struct dl_list **list, void *elem, Boolean (*compare_func)());


#if PS
extern int xdvi_temp_fd(char **tempfilename);
#endif
struct xchild; /* forward declaration */
extern void handle_child_exit(int status, struct xchild *this);
extern char *read_child_error(int fd);
extern Boolean fork_process(_Xconst char *file, char *const argv[]);
extern void print_child_error(void);
extern void prep_fd(int fd, wide_bool noblock);

struct bitmap; /* forward declaration */
extern void alloc_bitmap(struct bitmap *);

extern char *my_realpath(const char *path, char *real);
#ifdef HAVE_REALPATH
#include <limits.h>
# define REALPATH realpath
#else
# define REALPATH my_realpath
#endif

char *expand_homedir(const char *path);
void set_dvi_name(const char *new_filename);
FILE *XFOPEN(const char *path, const char *mode);


#ifndef KPATHSEA
extern void *xmalloc(unsigned);
extern void *xrealloc(void *, unsigned);
extern char *xstrdup(_Xconst char *);
extern char *xmemdump(_Xconst char *, size_t);
extern void xputenv(_Xconst char *, _Xconst char *);
#endif
extern char *xstrcat(char *str1, _Xconst char *str2);
extern int memicmp(const char *, const char *, size_t);
extern int xpipe(int *);
extern void close_a_file(void);
extern unsigned long num(FILE *, int);
extern long snum(FILE *, int);

extern void xdvi_exit(int);
extern void do_abort(void);

/* various levels of warning/error messages */
extern void xdvi_info(_Xconst char *fmt, ...);
extern void xdvi_warning(_Xconst char *fmt, ...);
extern void xdvi_error(_Xconst char *fmt, ...);
extern void xdvi_fatal(_Xconst char *fmt, ...);
extern void xdvi_abort(_Xconst char *fmt, ...);

extern int pointerlocate(int *, int *);
extern unsigned long parse_debugging_string(const char *arg);
extern unsigned long parse_debugging_option(const char *ptr);

extern int get_avg_font_width(XFontStruct *font);
extern char **split_line(const char *line, char sep, size_t begin, size_t end, size_t *ret_items);

extern char *find_file(const char *filename, struct stat *statbuf, kpse_file_format_type pathinfo);
extern char *find_dvi_file(const char *filename, Boolean *tried_dvi_ext);

/*
  hashtable wrapper functions, mostly used by dvi-draw.c to
  map filenames to integers. This uses the hashtable implementation
  from kpathsea, which is reasonably fast.
*/

/*
  We use this dummy wrapper stuct, which we cast to void *, to get integer
  values into/from the hashtable (natively, kpahtsea only supports string
  values).
*/
struct str_int_hash_item {
    int value;
};

extern Boolean find_str_int_hash(hash_table_type *hashtable, const char *key, size_t *val);
void put_str_int_hash(hash_table_type *hashtable, const char *key, size_t val);

/* Various error reporting macros.
   The reasons why these are macros are:
   - possibility to use __FILE__, __LINE__
   - (more importantly:) gcc can't do type checking on generic vararg
     macros, but does so for the printf() functions.
*/

/*
 *	Print an informative message to stdout,
 *	unless the resource `hush_stdout' is set.
 */
#define XDVI_INFO(X) do {						\
	fprintf(stdout, "%s: Info: ", prog == NULL ? "Xdvi" : prog);	\
	fprintf X;							\
	fprintf(stdout, "\n");						\
	fflush(stdout); /* to make sure it doesn't get intermingled with stderr */ \
} while(0)


/*
 *	Print a warning message to stderr.
 *	This can't be shut off by `hush_stdout'.
 */
#define XDVI_WARNING(X) do {						\
	fprintf(stderr, "%s: Warning: ", prog == NULL ? "Xdvi" : prog);	\
	fprintf X;							\
	fprintf(stderr, "\n");						\
} while(0)

/*
 *	Print an error message to stderr.
 *	This can't be shut off by `hush_stdout'.
 */
#define XDVI_ERROR(X) do {						\
	fprintf(stderr, "%s: Error: ", prog == NULL ? "Xdvi" : prog);	\
	fprintf X;							\
	fprintf(stderr, "\n");						\
} while(0)

/*
 *	Print an error message and quit. Use this only with extreme care,
 *	if recovery is really impossible!
 */
#define XDVI_FATAL(X) do {							\
	fprintf(stderr, "%s: Fatal error: ", prog == NULL ? "Xdvi" : prog);	\
	fprintf X;								\
	fprintf(stderr, "\n%s: Exiting.\n", prog == NULL ? "Xdvi" : prog);	\
	xdvi_exit(EXIT_FAILURE);						\
} while(0) 

/*
 *	Print an error message and abort. Use this instead of assertions
 *	if you want to give a more informative message.
 */
#define XDVI_ABORT(X) do {					\
	fprintf(stderr, "%s: %s:%d: Shouldn't happen: ",	\
			prog == NULL ? "Xdvi" : prog,		\
			__FILE__, __LINE__);			\
	fprintf X;						\
	fprintf(stderr, "\n");					\
	do_abort();						\
} while(0)

#endif /* UTIL_H_ */
